module SpecJs
  class Runner
    LIB_DIR = File.dirname(__FILE__)
    JS_JAR  = File.expand_path(LIB_DIR + "/../vendor/js.jar")
    COMMAND = "java -jar #{JS_JAR} -opt -1 specs.build.js"
    CORE    = %w[json2 spec spec.equiv spec.matcher spec.mock]
    ENVJS   = %w[env.rhino jquery]

    IMAGES = {
      :failed  => File.dirname(__FILE__) + "/../../resources/fail.png",
      :error   => File.dirname(__FILE__) + "/../../resources/error.png",
      :pending => File.dirname(__FILE__) + "/../../resources/success.png",
      :passed  => File.dirname(__FILE__) + "/../../resources/success.png"
    }

    TITLES = {
      :failed  => "Failed!",
      :passed  => "Passed!",
      :pending => "Passed!",
      :error   => "Error!"
    }

    attr_accessor :options, :output, :started, :finished

    def initialize(options = {})
      @options = options
      @started = options.delete(:started)
    end

    def run
      contents = String.new.tap do |code|
        code << "// THIS FILE IS AUTOMATICALLY GENERATED AND WILL BE OVERWRITTEN; DO NOT EDIT IT DIRECTLY.\n"
        code << dependencies
        options[:files].each {|path| code << File.read(path) << "\n\n"}
      end

      File.open("specs.build.js", "w+") {|f| f << contents}
      self.output = IO.popen(COMMAND) {|io| io.read}
      File.unlink("specs.build.js")
      dump
    end

    def stats
      @stats ||= begin
        result = {
          "examples" => 0,
          "failed"   => 0,
          "pending"  => 0,
          "errors"   => 0
        }

        copy = output.dup.gsub(/\e\[\d+(;\d+)?m/, "")
        copy.scan(/\d+ examples, \d+ failed.*?$/) do |match|
          match.scan(/(\d+) (examples|failed|pending|errors)/) do |data|
            result[data.last] += data.first.to_i
          end
        end

        self.output = copy unless options[:color]

        output.gsub!(/^.*?\d+ examples, \d+ failed.*?$\n\n?/, "")
        result
      end
    end

    def color(code, text)
      if options[:color]
        %(\e[#{code}#{text}\e[0m)
      else
        text
      end
    end

    def dump
      if stats["errors"].nonzero?
        status = :error
      elsif stats["failed"].nonzero?
        status = :failed
      elsif stats["pending"].nonzero?
        status = :pending
      else
        status = :passed
      end

      @finished = Time.now.to_f

      output << "Finished in %.2f seconds\n" % (finished - started)
      message = %[#{stats["examples"]} examples, #{stats["failed"]} failed]
      message << %[, #{stats["pending"]} pending] if stats["pending"].nonzero?
      message << %[, #{stats["errors"]} errors] if stats["errors"].nonzero?

      Notifier.notify(
        :image   => IMAGES[status],
        :title   => TITLES[status],
        :message => message
      )

      case status
      when :error, :failed
        output << color("31m", message)
      when :pending
        output << color("33m", message)
      else
        output << color("32m", message)
      end

      puts output

      if [:error, :failed].include?(status)
        exit 1
      else
        exit 0
      end
    end

    def dependencies
      libs = CORE
      libs += ENVJS unless options[:no_envjs]

      String.new.tap do |source|
        libs.each do |lib|
          source << %[load("#{LIB_DIR}/#{lib}.js");\n]
        end
      end
    end
  end
end
