function Spec(name, runner) {
	if (this.init) {
		this.init();
		this.name = name;
		this.runner = runner;
	} else {
		var suite = new Spec(name, runner);
		suite.run();
		return suite;
	}
};

// Specify the current suite that is being executed.
Spec.currentSuite = null;

// Specify either or not expectations can be defined on
// non existing functions.
Spec.stubNonExistentFunction = true;

// Initialize suite runner.
Spec.prototype.init = function() {
	this.name 	= null;
	this.runner = null;
	this.specs = [];
	this.stats = {specs: 0, passed: 0, pending: 0, failed: 0, error: 0};
	this.callbacks = {
		"before": 		[],
		"beforeAll": 	[],
		"after": 		[],
		"afterAll": 	[]
	};
};

Spec.prototype.print = function(text) {
	print(text);
};

Spec.prototype.yellow = function(text) {
	return "\033[0;33m" + text + "\033[0m";
};

Spec.prototype.green = function(text) {
	return "\033[0;32m" + text + "\033[0m";
};

Spec.prototype.red = function(text) {
	return "\033[0;31m" + text + "\033[0m";
};

Spec.prototype.gray = function(text) {
	return "\033[1;30m" + text + "\033[0m";
};

Spec.prototype.runCallbacks = function(name) {
	for (var i in this.callbacks[name]) {
		this.callbacks[name][i].call(this);
	};
};

Spec.prototype.report = function() {
	var spec,
		message,
		color,
		suite = Spec.currentSuite;
	
	this.print("\n" + suite.name);
	
	for (var i in suite.specs) {
		spec = suite.specs[i];
		message = "";
		
		switch (spec.result.name) {
			case "PendingError":
				message += this.yellow(spec.name + " " + spec.result.message);
				break;
			case "PassingError":
				message += this.green(spec.name);
				break;
			default:
				message += this.red(spec.name);
				message += "\n";
				message += this.gray("    # " + spec.result.message);
		}
		
		this.print("  "  + message);
	};
	
	message = "";
	message += this.stats.specs + " examples, ";
	message += this.stats.failed + " failed";
	
	if (this.stats.pending) {
		message += ", " + this.stats.pending + " pending";
	};
	
	if (this.stats.error) {
		message += ", " + this.stats.error + " errors";
	};
	
	if (this.stats.passed == this.stats.specs) {
		color = "green";
	} else if (this.stats.failed || this.stats.error) {
		color = "red";
	} else {
		color = "yellow";
	}
	
	print("\n" + this[color](message));
};

Spec.prototype.run = function() {
	var spec, errorMessage;
	
	Spec.currentSuite = this;
	this.runner.call(this);
	this.runCallbacks("beforeAll");
	
	for (var i in this.specs) {
		spec = this.specs[i];
		var result;
		
		try {
			this.stats.specs += 1;
			Spec.Matcher.count = 0;
			this.runCallbacks("before");
			spec.callback();
			
			if (Spec.Matcher.count) {
				result = {name: "PassingError", message: "Passed"};
			} else {
				pending("Your example didn't make any expectation");
			};
			
			this.runCallbacks("after");
		} catch (error) {
			if (typeof error == "string") {
				result = {message: error};
			} else {
				errorMessage = error.message.toString();
				
				if (error.fileName) {
					errorMessage += "\n    # File: " + error.fileName.toString();
				};
				
				if (error.lineNumber) {
					errorMessage += "\n    # Line: " + error.lineNumber.toString();
				};
				
				result = {
					message: errorMessage,
					name: error.name
				};
			};
		}
		
		Spec.Mock.unstubAll();
		
		switch (result.name) {
			case "PendingError":
				this.stats.pending += 1;
				break;
			case "PassingError":
				this.stats.passed += 1;
				break;
			case "FailingError":
				this.stats.failed += 1;
				break;
			default:
				this.stats.error += 1;
		};
		
		spec.result = result;
	};
	
	this.runCallbacks("afterAll");
	this.report();
};

// TOP LEVEL FUNCTIONS
function expect(subject) {
	var expectation = new Spec.Matcher(subject, true);
	expectation.not = new Spec.Matcher(subject, false);
	return expectation;
};

function before(callback) {
	Spec.currentSuite.callbacks["before"].push(callback);
};

function after(callback) {
	Spec.currentSuite.callbacks["after"].push(callback);
};

function beforeAll(callback) {
	Spec.currentSuite.callbacks["beforeAll"].push(callback);
};

function afterAll(callback) {
	Spec.currentSuite.callbacks["afterAll"].push(callback);
};

function specify(name, callback) {
	if (!callback) {
		callback = function() { pending("Not Yet Implemented"); };
	};
	
	Spec.currentSuite.specs.push({
		"name": name, 
		"callback": callback
	});
};

function it(name, callback) {
	specify("it " + name, callback);
};

function should(name, callback) {
	it("should " + name, callback);
};

function pending(reason) {
	throw {
		name: "PendingError",
		message: (reason? "[PENDING: " + reason + "]" : "[PENDING: No reason given]")
	};
};

function stub(object, attribute, value) {
	Spec.Mock.stub(object, attribute, value);
};

function unstub(object) {
	Spec.Mock.unstub(object);
};

function spy(object, func, args) {
	Spec.Mock.spy(object, func, args);
};

function describe(name, callback) {
	Spec(name, callback);
};

// Aliases
this.after_all = afterAll;
this.before_all = beforeAll;
