Spec.Matcher = function(subject, comparison) {
	this.subject = subject;
	this.comparison = comparison;
};

Spec.Matcher.count = 0;

Spec.Matcher.prototype.check = function(result, message, data) {
	Spec.Matcher.count += 1;
	
	if (result == !this.comparison) {
		throw {
			message: this.parseMessage(message, data), 
			status: "failed", 
			name: "FailingError"
		};
	};
};

Spec.Matcher.prototype.parseMessage = function(message, data) {
	data = data || {};
	data.subject = this.subject;
	
	return message.replace(/%\{(.*?)\}/gm, function(match, name){
		try {
			if (data[name] instanceof Array) {
				return "[" + data[name].join(",") + "]";
			} else {
				return data[name];
			};
		} catch (e) {
			return data[name];
		}
	});
};

Spec.Matcher.prototype.toBe = function(expected) {
	this.check(
		this.subject === expected,
		"expect %{subject} to be %{expected}",
		{expected: expected}
	);
};

Spec.Matcher.prototype.toBeEqualTo = function(expected) {
	this.check(
		Spec.equiv(this.subject, expected), 
		"expected %{subject} to be equal to %{expected}",
		{expected: expected}
	);
};

Spec.Matcher.prototype.toBeInstanceOf = function(klass) {
	var result = false;
	
	switch (typeof this.subject) {
		case "string":
			result = klass === String;
			break;
		case "number":
			result = klass === Number;
			break;
		default:
			result = this.subject instanceof klass;
	};
	
	this.check(result, "expected %{subject} to be an instance of %{klass}");
};

Spec.Matcher.prototype.toHaveLength = function(length) {
	this.check(
		typeof length == "number" && this.subject.length == length,
		"expected %{subject} to have length of %{length}",
		{length: length}
	);
};

Spec.Matcher.prototype.toHaveAttr = function(name, value) {
	var result = false;
	
	for (var property in this.subject) {
		if (property == name) {
			result = true;
			
			switch (arguments.length) {
				case 1:
					break;
				default:
					this.check(
						Spec.equiv(this.subject[name], value),
						"expected %{subject} to have %{name} property with %{value} as value",
						{name: name, value: value}
					);
			}
			
			break;
		};
	};
	
	this.check(
		result,
		"expected %{subject} to have %{name} attribute",
		{name: name}
	);
};

Spec.Matcher.prototype.toRespondTo = function(name) {
	this.check(
		typeof(this.subject[name]) == "function",
		"expected %{subject} to respond to %{name}",
		{name: name}
	);
};

Spec.Matcher.prototype.toBeNull = function() {
	this.check(
		this.subject === null, 
		"expected %{subject} to be null"
	);
};

Spec.Matcher.prototype.toBeUndefined = function() {
	this.check(
		this.subject === undefined, 
		"expected %{subject} to be undefined"
	);
};

Spec.Matcher.prototype.toBeDefined = function() {
	this.check(
		this.subject != undefined,
		"expected %{subject} to be defined");
};

Spec.Matcher.prototype.toMatch = function(regex) {
	this.check(
		regex.test(this.subject), 
		"expected %{subject} to match " + regex
	);
};

Spec.Matcher.prototype.toBeFalse = function() {
	this.check(
		!this.subject, 
		"expected %{subject} to be evaluated as false"
	);
};

Spec.Matcher.prototype.toBeTrue = function() {
	this.check(
		!!this.subject, 
		"expected %{subject} to be evaluated as false"
	);
};

Spec.Matcher.prototype.toContain = function(match) {
	if (typeof this.subject == "string") {
		this.check(
			this.subject.indexOf(match) != -1, 
			"expected %{subject} to include %{match}",
			{match: match}
		);
	} else if (this.subject instanceof Array) {
		for (var i = 0; i < this.subject.length; i++) {
			if (this.subject[i] == match) {
				return this.check(true);
			};
		}
		
		this.check(
			false, 
			"expected %{subject} to include %{match}",
			{match: match}
		);
	};
};

Spec.Matcher.prototype.toBeEmpty = function() {
	if (typeof this.subject == "string") {
		this.check(
			this.subject == "", 
			"expected %{subject} to be empty"
		);
	} else if (this.subject instanceof Array) {
		this.check(
			this.subject.length == 0, 
			"expected %{subject} to be empty"
		);
	} else {
		throw "toBeEmpty expects subject to be either a string or array";
	};
};

Spec.Matcher.prototype.toThrow = function(message) {
	var error;
	
	try {
		this.subject.call();
	} catch (e) {
		error = e;
	}
	
	this.check(
		error && (message == undefined || error.message == message || error == message),
		"expected %{subject} to throw error, but it didn't"
	);
};

Spec.Matcher.prototype.toHaveTag = function(selector, expected) {
	this.subject = jQuery(this.subject).find(selector);
	
	switch (typeof expected) {
		case "number":
			this.toHaveLength(expected);
			break;
		case "string":
			this.toHaveText(expected);
			break;
		default:
			this.subject = jQuery(this.subject).text();
			this.toMatch(expected);
	};
};

Spec.Matcher.prototype.toHaveText = function(expected) {
	var text = jQuery(this.subject).text();
	
	this.check(
		text == expected,
		"expected %{subject} to have %{expected} as text but it was %{actual}",
		{expected: expected, actual: text}
	);
};

Spec.Matcher.prototype.toBeGreaterThan = function(expected) {
	this.check(
		this.subject > expected, 
		"expected %{subject} to be greater than %{expected}",
		{expected: expected}
	);
};

Spec.Matcher.prototype.toBeLessThan = function(expected) {
	this.check(
		this.subject < expected, 
		"expected %{subject} to be less than %{expected}",
		{expected: expected}
	);
};

Spec.Matcher.prototype.toBeWithin = function(min, max) {
	this.check(
		this.subject >= min && this.subject <= max, 
		"expected %{subject} to be within %{min} and %{max}",
		{min: min, max: max}
	);
};

Spec.Matcher.prototype.toHaveValue = function(expected) {
	var actual = jQuery(this.subject).val();
	
	this.check(
		actual == expected,
		"expected %{subject} to have %{expected} as value, but was %{actual}",
		{expected: expected, actual: actual}
	);
};

Spec.Matcher.prototype.toHaveBeenCalledWith = function() {
	if (typeof this.subject != "function") {
		throw {
			name: "ArgumentError",
			message: "expected subject to be a function"
		};
	};
	
	if (typeof this.subject.stubId != "number") {
		throw {
			name: "ArgumentError",
			message: "expected function's owner to be spied"
		};
	};
	
	var object = Spec.Mock.objects[this.subject.stubId],
		methodName = this.subject.methodName,
		expected = [],
		calls = object.stubs[methodName].calls,
		message = null,
		i;
	
	for (i = 0; i < arguments.length; i++) {
		expected.push(arguments[i]);
	};
	
	for (i = 0; i < calls.length; i++) {
		if (Spec.equiv(expected, calls[i])) {
			return this.check(true);
		};
	};
	
	if (expected.length == 0) {
		message = "expected " + methodName + "() to have been called without arguments";
	} else {
		message = "expected " + methodName + "() to have been called with %{args}";
	};
	
	this.check(
		false,
		message,
		{args: expected}
	);
	
	return this;
};

// Aliases
Spec.Matcher.prototype.toHaveBeenCalled = Spec.Matcher.prototype.toHaveBeenCalledWith;

for (var name in Spec.Matcher.prototype) {
	if (!name.match(/^to[A-Z]/)) {
		continue;
	};
	
	var snakedName = name.replace(/([A-Z])/g, function(match, letter){
		return "_" + letter.toLowerCase();
	});
	
	Spec.Matcher.prototype[snakedName] = Spec.Matcher.prototype[name];
};
